#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define BYTE	unsigned char

#define	WIDTH	352
#define HEIGHT	288

/* quick and dirty declaration */
void bayer2rgb24(unsigned char *dst, unsigned char *src, long int width, long int height);

#include "sonix_compress.h"


static BYTE buf[101504];
static BYTE buf2[101504];
static BYTE buf3[101504*3];

/* reads file <name> into inp and returns size */
static int read_file(char *name, BYTE *inp)
{
	FILE	*file;
	int		filesize;

	fprintf(stderr, "opening %s\n", name);

	file = fopen(name, "rb");
	if (file == NULL) {
		fprintf(stderr, "fopen failed\n");
		return -1;
	}

	if (fseek(file, 0, SEEK_END) != 0) {
		fprintf(stderr, "fseek failed\n");
		return -1;
	}

	filesize = ftell(file);
	fprintf(stderr, "filesize = %d\n", filesize);

	if (fseek(file, 0, SEEK_SET) != 0) {
		fprintf(stderr, "fseek failed\n");
		return -1;
	}
	if (fread(inp, 1, filesize, file) != filesize) {
		fprintf(stderr, "fread failed\n");
		return -1;
	}
	fclose(file);

	return filesize;
}


int main(int argc, char *argv[])
{
	BYTE frame_header[] = {0xFF, 0xFF, 0x00, 0xC4, 0xC4, 0x96, 0x00};
	int i;
	BYTE *bufp;
	FILE *out_file;

	if (argc != 3) {
		fprintf(stderr, "Test util for SN9C101 image decompression\n");
		fprintf(stderr, "Usage: %s <compressed file> <decompressed file>\n", argv[0]);
		return -1;
	}

	if (read_file(argv[1], buf) < 0) {
		fprintf(stderr, "read_file failed\n");
		return -1;
	}

	sonix_decompress_init();

	bufp = buf;
	if (memcmp(buf, frame_header, sizeof(frame_header)) == 0) {
		/* show header */
		for (i = 0; i < 12; i++) {
			fprintf(stderr, " %02X", buf[i]);
		}
		fprintf(stderr, "\n");
		bufp += 12;
	}

	out_file = fopen(argv[2], "wb");
	if (out_file == NULL) {
		fprintf(stderr, "Failed to open '%s'\n!", argv[2]);
		return -1;
	}
	else {
/* '#if 1' decompresses a frame into a bayer PGM
   '#if 0' decompresses a frame into a color PNM
*/
#if 0
		sonix_decompress(WIDTH, HEIGHT, bufp, buf2);
		fprintf(out_file, "P5\n# SN9C101 decompressed image\n%d %d\n255\n", WIDTH, HEIGHT);
		fwrite(buf2, WIDTH*HEIGHT, 1, out_file);
#else
		sonix_decompress(WIDTH, HEIGHT, bufp, buf2);
		bayer2rgb24(buf3, buf2, WIDTH, HEIGHT);
		fprintf(out_file, "P6\n# SN9C101 decompressed image\n%d %d\n255\n", WIDTH, HEIGHT);
		fwrite(buf3, WIDTH*HEIGHT*3, 1, out_file);
#endif
		fclose(out_file);
	}

	return 0;
}
